@@ -19,6 +19,7 @@
 #define BADNESS_AVOID -300
 
 extern int enable_debug;
+void sanitize(char* s);
 
 struct procinfo {
     int oom_score;
@@ -52,8 +53,9 @@ static void maybe_notify(char* notif_command, char* notif_args)
         return;
 
     char notif[600];
-    snprintf(notif, 600, "%s %s", notif_command, notif_args);
-    system(notif);
+    snprintf(notif, sizeof(notif), "%s %s", notif_command, notif_args);
+    if (system(notif) != 0)
+        fprintf(stderr, "system(%s) failed: %d: %s\n", notif, errno, strerror(errno));
 }
 
 const char* const fopen_msg = "fopen %s failed: %s\n";
@@ -75,7 +77,8 @@ static struct procinfo get_process_stats(int pid)
         p.exited = 1;
         return p;
     }
-    fscanf(f, "%d", &(p.oom_score));
+    if (fscanf(f, "%d", &(p.oom_score)) < 1)
+        fprintf(stderr, "fscanf() oom_score failed: %d: %s\n", errno, strerror(errno));
     fclose(f);
 
     // Read /proc/[pid]/oom_score_adj
@@ -86,7 +89,9 @@ static struct procinfo get_process_stats(int pid)
         p.exited = 1;
         return p;
     }
-    fscanf(f, "%d", &(p.oom_score_adj));
+    if (fscanf(f, "%d", &(p.oom_score_adj)) < 1)
+        fprintf(stderr, "fscanf() oom_score_adj failed: %d: %s\n", errno, strerror(errno));
+
     fclose(f);
 
     // Read VmRss from /proc/[pid]/statm
@@ -97,7 +102,9 @@ static struct procinfo get_process_stats(int pid)
         p.exited = 1;
         return p;
     }
-    fscanf(f, "%*u %lu", &(p.vm_rss));
+    if (fscanf(f, "%*u %lu", &(p.vm_rss)) < 1)
+        fprintf(stderr, "fscanf() vm_rss failed: %d: %s\n", errno, strerror(errno));
+
     fclose(f);
 
     return p;
@@ -117,6 +124,7 @@ static void userspace_kill(DIR* procdir, int sig, int ignore_oom_score_adj,
     int victim_badness = 0;
     unsigned long victim_vm_rss = 0;
     char name[PATH_MAX];
+    char victim_name[PATH_MAX] = { 0 };
     struct procinfo p;
     int badness;
 
@@ -126,7 +134,7 @@ static void userspace_kill(DIR* procdir, int sig, int ignore_oom_score_adj,
         d = readdir(procdir);
         if (d == NULL) {
             if (errno != 0)
-                perror("readdir returned error");
+                perror("userspace_kill: readdir error");
 
             break;
         }
@@ -138,7 +146,7 @@ static void userspace_kill(DIR* procdir, int sig, int ignore_oom_score_adj,
 
         pid = strtoul(d->d_name, NULL, 10);
 
-        if (pid == 1)
+        if (pid <= 1)
             // Let's not kill init.
             continue;
 
@@ -148,6 +156,10 @@ static void userspace_kill(DIR* procdir, int sig, int ignore_oom_score_adj,
             // Process may have died in the meantime
             continue;
 
+        if (p.vm_rss == 0)
+            // Skip kernel threads
+            continue;
+
         badness = p.oom_score;
         if (ignore_oom_score_adj && p.oom_score_adj > 0)
             badness -= p.oom_score_adj;
@@ -156,7 +168,9 @@ static void userspace_kill(DIR* procdir, int sig, int ignore_oom_score_adj,
         snprintf(buf, sizeof(buf), "%d/stat", pid);
         FILE* stat = fopen(buf, "r");
         if (stat) {
-            fscanf(stat, "%*d (%[^)]s", name);
+            if (fscanf(stat, "%*d (%[^)]s", name) < 1)
+                fprintf(stderr, "fscanf() stat name failed: %d: %s\n", errno, strerror(errno));
+
             fclose(stat);
         } else {
             perror("could not read process name");
@@ -176,39 +190,49 @@ static void userspace_kill(DIR* procdir, int sig, int ignore_oom_score_adj,
             victim_pid = pid;
             victim_badness = badness;
             victim_vm_rss = p.vm_rss;
+            strncpy(victim_name, name, sizeof(victim_name) - 1);
             if (enable_debug)
                 printf("    ^ new victim (higher badness)\n");
         } else if (badness == victim_badness && p.vm_rss > victim_vm_rss) {
             victim_pid = pid;
             victim_vm_rss = p.vm_rss;
+            strncpy(victim_name, name, sizeof(victim_name) - 1);
             if (enable_debug)
                 printf("    ^ new victim (higher vm_rss)\n");
         }
-    }
+    } // end of while(1) loop
 
     if (victim_pid == 0) {
-        fprintf(stderr, "Error: Could not find a process to kill. Sleeping 10 seconds.\n");
-        maybe_notify(notif_command, "-i dialog-error 'earlyoom' 'Error: Could not find a process to kill'");
-        sleep(10);
+        fprintf(stderr, "Error: Could not find a process to kill. Sleeping 1 second.\n");
+        maybe_notify(notif_command, "-i dialog-error 'earlyoom' 'Error: Could not find a process to kill. Sleeping 1 second.'");
+        sleep(1);
         return;
     }
 
     if (sig != 0) {
-        fprintf(stderr, "Killing process %d %s\n", victim_pid, name);
+        fprintf(stderr, "Killing process: %s, pid: %d, badness: %d, VmRSS: %lu MiB\n",
+            victim_name, victim_pid, victim_badness, victim_vm_rss / 1024);
+    }
 
-        char notif_args[200];
-        snprintf(notif_args, 200, "-i dialog-warning 'earlyoom' 'Killing process %d %s'", victim_pid, name);
+    int res = kill(victim_pid, sig);
+
+    // Send the GUI notification AFTER killing a process. This makes it more likely
+    // that there is enough memory to spawn the notification helper.
+    if (sig != 0) {
+        char notif_args[PATH_MAX + 1000];
+        // maybe_notify() calls system(). We must sanitize the strings we pass.
+        sanitize(victim_name);
+        snprintf(notif_args, sizeof(notif_args), "-i dialog-warning 'earlyoom' 'Killing process %d %s'", victim_pid, victim_name);
         maybe_notify(notif_command, notif_args);
     }
 
-    if (kill(victim_pid, sig) != 0) {
-        perror("Could not kill process");
-        // Killing the process may have failed because we are not running as root.
-        // In that case, trying again in 100ms will just yield the same error.
-        // Throttle ourselves to not spam the log.
-        fprintf(stderr, "Sleeping 10 seconds\n");
-        maybe_notify(notif_command, "-i dialog-error 'earlyoom' 'Error: Failed to kill process'");
-        sleep(10);
+    // Killing the process may have failed because we are not running as root.
+    // In that case, trying again in 100ms will just yield the same error.
+    // Throttle ourselves to not spam the log.
+    if (res != 0) {
+        perror("userspace_kill: kill() failed, sleeping 1 second");
+        maybe_notify(notif_command, "-i dialog-error 'earlyoom' 'Error: Failed to kill process. Sleeping 1 second.'");
+        sleep(1);
     }
 }
 
